# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Mads Chr. Olesen <mchro@cs.aau.dk>
#This program is free software: you can redistribute it and/or modify it 
#under the terms of the GNU General Public License version 3, as published 
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but 
#WITHOUT ANY WARRANTY; without even the implied warranties of 
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along 
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from opaal import util

class IllegalExpressionException(util.Error):
    pass

def expression_to_python_DEPR(expr):
    """Rewrite some common expression syntax to something python compatible.
    Deprecated."""
    replacements = {
        'true': 'True',
        'false': 'False',
        '&&': ' and ',
        ',': ' ; ',
    }
    for (s, r) in replacements.iteritems():
        expr = expr.replace(s, r)
    return expr

def expression_to_python(node):
    binops = {
        'Plus': '+',
        'Minus': '-',
        'Times': '*',
        'Divide': '/',
    }
    if node.type in ['Identifier', 'Number']:
        return str(node.leaf)
    elif node.type in binops.keys():
        return expression_to_python(node.children[0]) + \
            binops[node.type] + \
            expression_to_python(node.children[1])
    elif node.type == 'Expression':
        return expression_to_python(node.children[0])

def goal_expression_to_python(succgen, node):
    if node.type == 'And':
        return goal_expression_to_python(succgen, node.children[0]) + \
            " and " + goal_expression_to_python(succgen, node.children[1])
    elif node.type == 'Or':
        return goal_expression_to_python(succgen, node.children[0]) + \
            " or " + goal_expression_to_python(succgen, node.children[1])
    elif node.type in ['Greater', 'Less', 'LessEqual', 'GreaterEqual', 'Equal', 'Minus']:
        op = {'Greater': ' > ',
              'GreaterEqual': ' >= ',
              'Less': ' < ',
              'LessEqual': ' <= ',
              'Equal': ' == ',
              'Minus': ' - ',
              }[node.type]
        return goal_expression_to_python(succgen, node.children[0]) + \
             op + goal_expression_to_python(succgen, node.children[1])
    elif node.type == 'Number' and len(node.children) == 0:
        return str(node.leaf)
    elif node.type == 'Identifier' and len(node.children) == 0:
        i_name = node.leaf
        if i_name in [name for (name, _, _) in succgen.variables] or \
            i_name in [name for (name, _) in succgen.clocks]:
            return node.leaf
        elif i_name in ['True', 'False']:
            return i_name
        elif i_name in succgen.constants:
            return i_name
        raise IllegalExpressionException('Unknown identifier ' + i_name)
    elif node.type == 'Identifier' and len(node.children) == 1:
        #XXX when do we not have an identifier here? when doing index'es
        #assert node.children[0].type == 'Identifier'


        #Of the form "Template."
        if node.leaf in succgen.templatename_to_idx:
            t_idx = succgen.templatename_to_idx[node.leaf]
            template = succgen.model.templates[t_idx]
            f_name = node.children[0].leaf

            #case: Template.var | Template.clock
            #XXX this is incorrect
            if f_name in [name for (name, _, _) in succgen.variables]:
                return f_name
            if f_name in [name for (name, _) in succgen.clocks]:
                return f_name

            #case: Template.location
            for (loc, loc_idx) in zip(template.locations,
                range(len(template.locations))):
                if f_name == loc.name.value:
                    return "locs[%d] == %d" % (t_idx, loc_idx)
        elif node.leaf in [v[0] for v in succgen.variables] or node.leaf in [c[0] for c in succgen.clocks]:
            if node.children[0].type == 'Identifier':
                return node.leaf + "." + node.children[0].leaf
            elif node.children[0].type == 'Index':
                return node.leaf + "[" + goal_expression_to_python(succgen, node.children[0].leaf) + "]"
            else:
                raise IllegalExpressionException('Unknown construction ' + 
                        node.leaf + " with child of type " + node.children[0].typo)
        else:
            raise IllegalExpressionException('Unknown identifier ' + 
                node.leaf + "." + node.children[0].leaf)

    elif node.type == 'FunctionCall' and len(node.children) == 1:
        #XXX we do not handle function parameters
        assert node.children[0].type == 'Identifier'

        #XXX Assume of the form "variable.f()"
        return node.children[0].leaf + "." + node.children[0].children[0].leaf + "()"
        
    

    return ""

# vim:ts=4:sw=4:expandtab
