# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Mads Chr. Olesen <mchro@cs.aau.dk>
#This program is free software: you can redistribute it and/or modify it 
#under the terms of the GNU General Public License version 3, as published 
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but 
#WITHOUT ANY WARRANTY; without even the implied warranties of 
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along 
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import logging
logger = logging.getLogger('simple_pwlist')

from opaal import util

class NoMoreStatesException(util.Error):
    pass

class GoalFoundException(util.Error):
    pass

class SimplePWList:
    def __init__(self):
        self.waiting = []
        self.passed_waiting = set() 
        #number of successful adds to passed-waiting list
        self.num_adds = 0
        self.wp = {}

    def stateInPwList(self, state):
        return state in self.passed_waiting
        #return state in self.wp

    def getWP(self, state):
        return self.wp[state]

    def setWP(self, state, w, p):
        self.wp[state]=(w, p)

    def addState(self, state, w, p):
        """Add a state to the passed-waiting list.
        @returns: state if state was new, False if state was seen before"""
        self.num_adds += 1

        try:
            (w1, p1) = self.wp[state]
#New minimum values 
            w2 = w1
            p2 = p1
            update = False

            if w < w2:
                update = True
                w2 = w
            if p < p2:
                update = True
                p2 = p

            #New value is
            if update:
                self.wp[state] = (w2, p2)	

            #If there are something unexplored and state is not already on pw list            
            #if w2 < p2 and not w1 < p1:
            if w2 < p2: #Add if unexplored
                self.waiting.append(state)
                return state
            else:
                return False

        except KeyError:
            self.waiting.append(state)
            self.passed_waiting.add(state)
	    self.wp[state] = (w, p)
            return state

            
    def getWaitingState(self):
        try:
            return self.waiting.pop()
        except IndexError:
            raise NoMoreStatesException()

    def lenOfPWList(self):
        return len(self.passed_waiting)

# vim:ts=4:sw=4:expandtab
